@@ -1,124 +1,121 @@
 package Bitcoin::Crypto::Network;
-$Bitcoin::Crypto::Network::VERSION = '1.008';
+$Bitcoin::Crypto::Network::VERSION = '2.000';
 use v5.10;
 use strict;
 use warnings;
+
 use Moo;
-use Scalar::Util qw(blessed);
-use Types::Standard qw(Str Int);
-use Types::Common::String qw(StrLength);
+use Mooish::AttributeBuilder -standard;
+use Type::Params -sigs;
 
 use Bitcoin::Crypto::Exception;
+use Bitcoin::Crypto::Types qw(Object HashRef CodeRef Str StrLength Int);
 
 use namespace::clean;
 
 my %networks;
 my $default_network;
 
-has "id" => (
-	is => "ro",
+has param 'id' => (
 	isa => Str,
-	required => 1,
 );
 
-has "name" => (
-	is => "ro",
+has param 'name' => (
 	isa => Str,
-	required => 1,
 );
 
-has "p2pkh_byte" => (
-	is => "ro",
+has param 'p2pkh_byte' => (
 	isa => StrLength [1, 1],
-	required => 1,
 );
 
-has "wif_byte" => (
-	is => "ro",
+has param 'wif_byte' => (
 	isa => StrLength [1, 1],
-	required => 1,
 );
 
-has "p2sh_byte" => (
-	is => "ro",
+has param 'p2sh_byte' => (
 	isa => StrLength [1, 1],
 	required => 0,
 );
 
-has "segwit_hrp" => (
-	is => "ro",
+has param 'segwit_hrp' => (
 	isa => Str,
 	required => 0,
 );
 
-has "extprv_version" => (
-	is => "ro",
+has param 'extprv_version' => (
 	isa => Int,
 	required => 0,
 );
 
-has "extpub_version" => (
-	is => "ro",
+has param 'extpub_version' => (
 	isa => Int,
 	required => 0,
 );
 
-has "extprv_compat_version" => (
-	is => "ro",
+has param 'extprv_compat_version' => (
 	isa => Int,
 	required => 0,
 );
 
-has "extpub_compat_version" => (
-	is => "ro",
+has param 'extpub_compat_version' => (
 	isa => Int,
 	required => 0,
 );
 
-has "extprv_segwit_version" => (
-	is => "ro",
+has param 'extprv_segwit_version' => (
 	isa => Int,
 	required => 0,
 );
 
-has "extpub_segwit_version" => (
-	is => "ro",
+has param 'extpub_segwit_version' => (
 	isa => Int,
 	required => 0,
 );
 
-has "bip44_coin" => (
-	is => "ro",
+has param 'bip44_coin' => (
 	isa => Int,
 	required => 0,
 );
 
+signature_for register => (
+	method => 1,
+	positional => [HashRef, {slurpy => 1}],
+);
+
 sub register
 {
-	my ($self, %config) = @_;
-	if (!blessed $self) {
-		$self = $self->new(%config);
+	my ($self, $config) = @_;
+
+	if (!ref $self) {
+		$self = $self->new($config);
 	}
+
 	$networks{$self->id} = $self;
 	return $self;
 }
 
+signature_for set_default => (
+	method => Object,
+	positional => [],
+);
+
 sub set_default
 {
 	my ($self) = @_;
 
 	Bitcoin::Crypto::Exception::NetworkConfig->raise(
-		"network must be an instance of Bitcoin::Crypto::Network"
-	) unless blessed($self);
-
-	Bitcoin::Crypto::Exception::NetworkConfig->raise(
-		"the network needs to be registered before becoming the default one"
+		'the network needs to be registered before becoming the default one'
 	) unless defined $networks{$self->id};
 
 	$default_network = $self->id;
 	return $self;
 }
 
+signature_for supports_segwit => (
+	method => Object,
+	positional => [],
+);
+
 sub supports_segwit
 {
 	my ($self) = @_;
@@ -126,6 +123,11 @@ sub supports_segwit
 	return defined $self->segwit_hrp;
 }
 
+signature_for find => (
+	method => Str,
+	positional => [CodeRef, {optional => 1}],
+);
+
 sub find
 {
 	my ($class, $sub) = @_;
@@ -133,18 +135,19 @@ sub find
 	return keys %networks
 		unless defined $sub;
 
-	Bitcoin::Crypto::Exception::NetworkConfig->raise(
-		"argument passed to Bitcoin::Crypto::Network->find must be a coderef returning boolean"
-	) unless ref $sub eq "CODE";
-
 	return grep { $sub->($networks{$_}) } keys %networks;
 }
 
+signature_for get => (
+	method => Str,
+	positional => [Str, {default => sub { $default_network }}],
+);
+
 sub get
 {
 	my ($class, $id) = @_;
 
-	my $network = $networks{$id // $default_network};
+	my $network = $networks{$id};
 	Bitcoin::Crypto::Exception::NetworkConfig->raise(
 		"network $id is not registered"
 	) unless defined $network;
@@ -162,12 +165,12 @@ sub get
 ### BITCOIN
 
 __PACKAGE__->register(
-	id => "bitcoin",
-	name => "Bitcoin Mainnet",
+	id => 'bitcoin',
+	name => 'Bitcoin Mainnet',
 	p2pkh_byte => "\x00",
 	p2sh_byte => "\x05",
 	wif_byte => "\x80",
-	segwit_hrp => "bc",
+	segwit_hrp => 'bc',
 
 	extprv_version => 0x0488ade4,
 	extpub_version => 0x0488b21e,
@@ -182,12 +185,12 @@ __PACKAGE__->register(
 )->set_default;
 
 __PACKAGE__->register(
-	id => "bitcoin_testnet",
-	name => "Bitcoin Testnet",
+	id => 'bitcoin_testnet',
+	name => 'Bitcoin Testnet',
 	p2pkh_byte => "\x6f",
 	p2sh_byte => "\xc4",
 	wif_byte => "\xef",
-	segwit_hrp => "tb",
+	segwit_hrp => 'tb',
 
 	extprv_version => 0x04358394,
 	extpub_version => 0x043587cf,
@@ -204,8 +207,8 @@ __PACKAGE__->register(
 ### DOGECOIN
 
 __PACKAGE__->register(
-	id => "dogecoin",
-	name => "Dogecoin Mainnet",
+	id => 'dogecoin',
+	name => 'Dogecoin Mainnet',
 	p2pkh_byte => "\x1e",
 	p2sh_byte => "\x16",
 	wif_byte => "\x9e",
@@ -217,8 +220,8 @@ __PACKAGE__->register(
 );
 
 __PACKAGE__->register(
-	id => "dogecoin_testnet",
-	name => "Dogecoin Testnet",
+	id => 'dogecoin_testnet',
+	name => 'Dogecoin Testnet',
 	p2pkh_byte => "\x71",
 	p2sh_byte => "\xc4",
 	wif_byte => "\xf1",
@@ -232,34 +235,35 @@ __PACKAGE__->register(
 1;
 
 __END__
+
 =head1 NAME
 
-Bitcoin::Crypto::Network - Management tool for cryptocurrency networks
+Bitcoin::Crypto::Network - Network management class
 
 =head1 SYNOPSIS
 
 	use Bitcoin::Crypto::Network;
 
-	# by default network is set to bitcoin
+	# the default network is Bitcoin Mainnet
 	# get() without arguments returns default network
 
-	Bitcoin::Crypto::Network->get->name; # Bitcoin Mainnet
+	Bitcoin::Crypto::Network->get->name; # 'Bitcoin Mainnet'
 
 	# by default there are two networks specified
 	# find() without arguments returns a list of all network ids
 
-	Bitcoin::Crypto::Network->find; # (mainnet, testnet)
+	Bitcoin::Crypto::Network->find; # list of strings
 
 	# you can get full network configuration with get() using network id
 
-	Bitcoin::Crypto::Network->get("bitcoin_testnet")->name; # Bitcoin Testnet
+	Bitcoin::Crypto::Network->get('bitcoin_testnet')->name; # 'Bitcoin Testnet'
 
 	# search for network and get array of keys in return
 	# there will be multiple results if your search is matched
 	# by multiple networks
 
-	Bitcoin::Crypto::Network->find(sub { shift->name eq "Bitcoin Mainnet" }); # (mainnet)
-	Bitcoin::Crypto::Network->find(sub { shift->p2pkh_byte eq "\x6f" }); # (testnet)
+	Bitcoin::Crypto::Network->find(sub { shift->name eq 'Bitcoin Mainnet' }); # ('bitcoin')
+	Bitcoin::Crypto::Network->find(sub { shift->p2pkh_byte eq "\x6f" }); # ('bitcoin_testnet')
 
 	# if you're working with cryptocurrency other than Bitcoin you need to add a new network
 
@@ -270,13 +274,13 @@ Bitcoin::Crypto::Network - Management tool for cryptocurrency networks
 	# register() can be used to create a network
 
 	my $litecoin = Bitcoin::Crypto::Network->register(
-		id => "litecoin",
-		name => "Litecoin Mainnet",
+		id => 'litecoin',
+		name => 'Litecoin Mainnet',
 		p2pkh_byte => "\x30",
 		wif_byte => "\xb0",
 	);
 
-	# after you've added your network you can set it as default. This means that
+	# after you've added a new network you can set it as a default. This means that
 	# all extended keys generated by other means than importing serialized key and
 	# all private keys generated by other means than importing WIF / extended keys
 	# will use that configuration.
@@ -286,13 +290,13 @@ Bitcoin::Crypto::Network - Management tool for cryptocurrency networks
 
 =head1 DESCRIPTION
 
-This package allows you to manage non-bitcoin cryptocurrencies.
-Before you start producing keys and addresses for your favorite crypto
-you have to configure its network first.
+This package allows you to manage non-bitcoin cryptocurrencies or chains other
+than mainnet. Before you start producing keys and addresses for your favorite
+crypto you have to configure its network first.
 
 =head1 PREDEFINED NETWORKS
 
-Here is a list of networks that are already defined and can be used without defining them.
+Here is a list of networks that are already defined and can be used out of the box.
 
 If you want to see more predefined networks added and you're willing to make
 some research to find out the correct values for the configuration fields,
@@ -316,11 +320,12 @@ defined with id: C<dogecoin_testnet>
 
 =head1 CONFIGURATION
 
-Right now networks only require four keys, which are marked with C<(*)>
+Configuration fields marked with C<(*)> are required. The rest are optional,
+but some functions of the system will refuse to work without them.
 
 	my %config = (
-		id             => "(*) identifier for the network",
-		name           => "(*) human-readable network name",
+		id             => "(*) string identifier for the network, eg. 'bitcoin'",
+		name           => "(*) human-readable network name, eg. 'Bitcoin Mainnet'",
 		p2pkh_byte     => "(*) p2pkh address prefix byte, eg. 0x00",
 		wif_byte       => "(*) WIF private key prefix byte, eg. 0x80",
 		p2sh_byte      => "p2sh address prefix byte, eg. 0x05",
@@ -336,16 +341,21 @@ Right now networks only require four keys, which are marked with C<(*)>
 		bip44_coin => "bip44 coin number, eg. 0",
 	);
 
-After you register a network with this hashref your program will be able to import keys for that
-network but all keys created from other sources will be treated as Bitcoin.
-You need to set_default to make all new keys use it. If you use many
-networks it might be better to set a network with key's set_network method:
+You can then C<register> this network:
 
-	$priv->set_network("network_id");
+	Bitcoin::Crypto::Network->register(%config);
 
-Remember that if you don't specify network field for some feature you won't be able to
-use it. For example the module will complain if you try to generate segwit address
-with custom network without segwit_hrp field set.
+Your program will now be able to import keys for that network but all keys
+created from other sources will be treated as the default (I<Bitcoin>). You
+need to C<set_default> to make all new keys use it. If your usage is not
+restrained to a single network, it might be better to set a network manually to
+a single key with its C<set_network> method:
+
+	$priv->set_network('network_id');
+
+Remember that if you don't specify network field for some feature you won't be
+able to use it. For example, the module will complain if you try to generate
+segwit address without C<segwit_hrp> field set.
 
 =head1 METHODS
 
@@ -354,9 +364,10 @@ with custom network without segwit_hrp field set.
 	$network_object = $class->register(%config)
 	$network_object = $object->register()
 
-Adds the network instance to a list of known networks.
+Adds a network instance to a list of known networks.
 
-Calls L</new> with keys present in C<%config> hash when called in static context.
+Calls L</new> with keys present in C<%config> hash when called in class
+context.
 
 Returns the network instance.
 
@@ -364,7 +375,8 @@ Returns the network instance.
 
 	$network_object = $object->set_default()
 
-Sets the network as default one. All newly created private and public keys will be bound to this network.
+Sets a network as the default one. All newly created private and public keys
+will be bound to this network.
 
 Returns the network instance.
 
@@ -372,21 +384,25 @@ Returns the network instance.
 
 	$bool = $object->supports_segwit()
 
-Returns a boolean which can be used to determine whether a given network has segwit configured.
+Returns a boolean which can be used to determine whether a given network has
+SegWit configured.
 
 =head2 new
 
 	$network_object = $class->new(%config)
 
-Creates a new network instance. See L</CONFIGURATION> for a list of possible C<%config> keys.
+Creates a new network instance. See L</CONFIGURATION> for a list of possible
+C<%config> keys.
 
 =head2 get
 
 	$network_object = $class->get($id = undef)
 
-Without arguments, returns the default network configuration as the Bitcoin::Crypto::Network instance.
+Without arguments, returns the default network configuration as the
+C<Bitcoin::Crypto::Network> instance.
 
-With the C<$id> argument (string), returns the instance of a configuration matching the id.
+With the C<$id> argument (string), returns the instance of a configuration
+matching the id.
 
 Throws an exception if network doesn't exist.
 
@@ -394,31 +410,24 @@ Throws an exception if network doesn't exist.
 
 	@network_objects = $class->find($sub = undef)
 
-Without arguments, returns a list of all registered network ids.
+Without arguments, returns a list of all registered network identifiers.
 
-With the C<$sub> argument (coderef), searches for all networks that pass the criteria and returns their ids.
-
-Returns list of network instances.
-
-The C<$sub> will be passed all the instances of registered networks, one at a time.
-
-If must perform required checks and return a boolean value. All the networks that pass this
-test will be returned. Example:
+With the C<$sub> argument (coderef), searches for all networks that pass the
+criteria and returns their ids. The C<$sub> will be passed all the instances of
+registered networks, one at a time. If must perform required checks and return
+a boolean value. All the networks that pass this test will be returned.
+Example:
 
 	sub {
 		my $instance = shift;
-		return $instance->name eq "Some name";
+		return $instance->name eq 'Some name';
 	}
 
+Returns a list of network instances (objects).
+
 =head1 SEE ALSO
 
-=over 2
+L<Bitcoin::Crypto::Key::ExtPrivate>
 
-=item L<Bitcoin::Crypto::Key::ExtPrivate>
-
-=item L<Bitcoin::Crypto::Key::Private>
-
-=back
-
-=cut
+L<Bitcoin::Crypto::Key::Private>
 
