@@ -1,26 +1,26 @@
 package Bitcoin::Crypto::Role::Network;
-$Bitcoin::Crypto::Role::Network::VERSION = '1.008';
+$Bitcoin::Crypto::Role::Network::VERSION = '2.000';
 use v5.10;
 use strict;
 use warnings;
-use Types::Standard qw(InstanceOf Str);
 use Scalar::Util qw(blessed);
+use Mooish::AttributeBuilder -standard;
 
+use Bitcoin::Crypto::Types qw(InstanceOf Str);
 use Bitcoin::Crypto::Network;
 use Bitcoin::Crypto::Exception;
 use Moo::Role;
 
-has "network" => (
-	is => "ro",
-	isa => (InstanceOf ["Bitcoin::Crypto::Network"])
+has param 'network' => (
+	coerce => (InstanceOf ['Bitcoin::Crypto::Network'])
 		->plus_coercions(Str, q{Bitcoin::Crypto::Network->get($_)}),
 	default => sub {
 		return Bitcoin::Crypto::Network->get;
 	},
-	coerce => 1,
-	writer => "_set_network"
+	writer => -hidden,
 );
 
+# make writer chainable
 sub set_network
 {
 	my ($self, $network) = @_;
